from pathlib import Path
import logging
from numpy import repeat,empty
from xarray import DataArray
from numpy import atleast_1d
from os import chdir
#
from sciencedates import datetime2gtd
#
import glowaurora
from glowfort import tselec,meters,csw,gtd7
glowpath=Path(glowaurora.__path__[0])/'..'
oldcwd = Path.cwd()

def rungtdGLOW(dtime,altkm,glat,glon,f107a,f107,ap,mass,tselecopts):
    chdir(str(glowpath))
    ap = atleast_1d(ap)
    if ap.size==1: ap = repeat(ap,7)
    species = ['He','O','N2','O2','Ar','Total','H','N','AnomalousO']
    ttypes = ['exotemp','heretemp']

    tselec(tselecopts) #like the msis_driver example
    logging.debug(f'tselec options used:   {csw.sw}') #don't use tretrv, it doesn't work

    iyd,utsec,stl = datetime2gtd(dtime,glon)

    altkm = atleast_1d(altkm)
    dens = empty((altkm.size,9)); temp=empty((altkm.size,2))

    meters(1) # makes output in m^-3 and kg/m^-3
    for i,a in enumerate(altkm):
        dens[i,:],temp[i,:] = gtd7(iyd,utsec,a,glat,glon,stl, f107a,f107, ap,mass)

    densd = DataArray(dens, dims=['z_km','species'],
                      coords={'z_km':altkm, 'species':species})
    tempd = DataArray(temp, dims=['z_km','temperature'],
                      coords={'z_km':altkm, 'temperature':ttypes})
    return densd,tempd



from pathlib import Path
import xarray
import logging
from numpy.ma import masked_invalid
from matplotlib.pyplot import figure, draw
from matplotlib.ticker import MultipleLocator #LogFormatterMathtext,
from matplotlib.colors import LogNorm

dymaj=50
dymin=10
dpi = 100

def _nicez(ax, params:dict):
    """ make ordinate axis look nice for altitude """
    ax.autoscale(True,axis='both',tight=True)
    ax.set_ylim(params['zlim'])
    ax.yaxis.set_major_locator(MultipleLocator(dymaj))
    ax.yaxis.set_minor_locator(MultipleLocator(dymin))
    ax.grid(True,which='major',linewidth=1.)
    ax.grid(True,which='minor',linewidth=0.5)
    ax.tick_params(axis='both',which='major',labelsize='medium')

def _plotisr(params,sim,st:str):

    if not 'photIon' in sim or not 'isr' in sim:
        return

    fg = figure(figsize=(15,8))
    axs = fg.subplots(1,2,sharey=True,)
    fg.suptitle(st)

    ind = ['nO','nO2','nN2','nNO']
    ax = axs[0]

    ax.semilogx(sim['photIon'].loc[:,ind],
                sim.z_km)
    ax.set_xlabel('Number Density')
    ax.set_xscale('log')
    ax.set_xlim(left=1e1)
    ax.set_ylabel('Altitude [km]')
    _nicez(ax,params)
    ax.legend(ind)
    ax.set_title('Neutral Number Density')

    ind=['Te','Ti']
    ax = axs[1]
    # Need to use .values here.
    ax.semilogx(sim['isr'].loc[:,ind],
                sim.z_km)
    ax.set_xlabel('Temperature [K]')
    ax.legend(ind)
    _nicez(ax,params)
    ax.set_xlim(100,10000)
    ax.set_title('Background Temperature')

    writeplots(fg,'bg_',params)

def _plotver(sim,params,supertitle):
    if not 'phitop' in sim:
        return

    fg = figure(figsize=(15,8))
    axs = fg.subplots(1,3,sharey=False)
    fg.suptitle(supertitle)
    fg.tight_layout(pad=3.2, w_pad=0.6)

#%% incident flux at top of ionosphere
    ax = axs[0]
    ax.plot(sim['phitop'].eV, sim['phitop'], marker='.')

    titxt=f'Total Incident Flux={params["flux"]:.1f},'
    ax.set_title(titxt)

    ax.set_xlabel('Beam Energy [eV]')
    ax.set_ylabel('Flux [erg sr$^{-1}$ s$^{-1}$]')
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_ylim(1e-4,1e6)
    ax.grid(True)
# %% ver visible
    if not 'ver' in sim:
        return

    ind= [4278, 5200, 5577, 6300]
    ax = axs[1]
    ax.plot(sim['ver'].loc[...,ind],
            sim.z_km)
    ax.set_xlabel('Volume Emission Rate')
    ax.set_ylabel('altitude [km]')
    _nicez(ax,params)
    ax.set_xscale('log')
    if not 'eig' in params['makeplot']:
        ax.set_xlim(1e-5,1e3)
    ax.legend(ind,loc='best')
    ax.set_title('Volume Emission Rate: Visible')
#%% ver invisible
    ind = [3371,7320,10400,3466,7774, 8446,3726,1356., 1304., 1027., 989., 1900.]
    ax = axs[2]
    ax.plot(sim['ver'].loc[...,ind],
            sim.z_km)
    ax.set_xlabel('Volume Emission Rate')
    _nicez(ax,params)
    ax.set_xscale('log')
    if not 'eig' in params['makeplot']:
        ax.set_xlim(1e-5,1e3)
    ax.legend(ind,loc='best')
    ax.set_title('Volume Emission Rate: IR & UV')

    writeplots(fg,'ver_',params)


def _plotdens(sim,params,supertitle):

    if not 'photIon' in sim:
        return

    ind=['ne','nO+(2P)','nO+(2D)','nO+(4S)','nN+','nN2+','nO2+','nNO+']
    fg=figure()
    ax = fg.gca()
    ax.semilogx(sim['photIon'].loc[:,ind],
                sim.z_km)
    ax.set_xlabel('Density')
    ax.set_xscale('log')
    ax.set_xlim(left=1e-3)
    _nicez(ax,params)
    ax.legend(ind)
    ax.set_title('Electron and Ion Densities')
    ax.set_ylabel('Altitude [km]')

    writeplots(fg,'effects_',params)


def _plotioniz(sim,params,supertitle):
    if 'photIon' not in sim:
        return

    fg = figure(figsize=(15,8))
    axs = fg.subplots(1,2,sharey=True,)
    fg.suptitle(supertitle)
    fg.tight_layout(pad=3.2, w_pad=0.3)

    ind=['photoIoniz','eImpactIoniz']
    ax = axs[0]
    ax.plot(sim['photIon'].loc[:,ind],
            sim.z_km)
    ax.set_xlabel('ionization')
    ax.set_xscale('log')
    ax.set_xlim(left=1e-1)
    _nicez(ax,params)
    ax.legend(ind)
    ax.set_title('Photo and e$^-$ impact ionization')
    ax.set_ylabel('Altitude [km]')

    ind=['O','O2','N2']
    ax = axs[1]
    ax.plot(sim['sion'].T,
            sim.z_km)
    ax.set_xscale('log')
    ax.set_xlim(1e-5,1e4)
    _nicez(ax,params)
    ax.set_xlabel('e$^-$ impact ioniz. rate')

    ax.set_title('electron impact ioniz. rates')
    ax.legend(ind)

    writeplots(fg,'ioniz_',params)


def plotaurora(params:dict, sim:xarray.Dataset):
    """ Plot all sorts of auroral/dayglow parameters from GLOW simulation. """
    makeplot = params['makeplot']
    if not makeplot:
        return

    if 'EK' in params:
        EK = params['EK']
    else:
        EK = params['E0']

    titlend = f'$E_0={EK}$ eV  SZA={sim.sza:.1f}$^\circ$'
    supertitle = f'{params["t0"]} ({params["glat"]},{params["glon"]}) \n {titlend}'

#%% neutral background (MSIS) and Te,Ti (IRI-90)
    if not 'eig' in makeplot:
        _plotisr(params,sim,supertitle)
#%% production and loss rates for species
    plotprodloss(sim['prates'], sim['lrates'], params, supertitle)
#%% volume emission rate
    _plotver(sim,params,supertitle)
# %% Ne, Ni
    if not 'eig' in makeplot:
        _plotdens(sim,params,supertitle)
# %% total energy deposition vs. altitude
    if not 'eig' in makeplot:
        plotenerdep(sim['tez'],params,titlend)
#% % e^- impact ionization rates from ETRANS
        _plotioniz(sim,params,supertitle)
#%% constituants of per-wavelength VER
    if not 'eig' in makeplot:
        _plotconstit(sim,params,supertitle)
\
def _plotconstit(sim,params,supertitle):
    # FIXME: talk to Stan about what these  mean, etc.
    if 'zceta' not in sim:
        return

    zcsum = sim['zceta'].sum(axis=-1)

    fg = figure(figsize=(15,8))
    axs = fg.subplots(3,4,sharey=True)
    for ax,zc,i in zip(axs.ravel(),
                       sim['zceta'].transpose('wavelength_nm','type','z_km'),
                       sim.wavelength_nm):
        ax.plot(zc.T,zc.z_km)
        ax.set_xscale('log')
        #ax.set_xlabel('emission constituants  ' + titlend)
        ax.set_ylabel('Altitude [km]')
        ax.set_title(f'{i.values} angstrom')
        #ax.legend(True)

    writeplots(fg,'constit_',params)


def plotenerdep(tez,params,titlend=''):
    """ plot energy deposition vs. altitude """
    fg= figure()
    ax = fg.gca()

    tez = tez.squeeze()

    if tez.ndim==1:
        ax.plot(tez, tez.z_km)
        ax.set_xscale('log')
        ax.set_xlim(1e-1,1e6)
        ax.set_xlabel('Energy Deposited')
    else:
        ax.set_label('Beam Energy [eV]')
        hi=ax.pcolormesh(tez.eV,tez.z_km,masked_invalid(tez.values),norm=LogNorm())
        cb=fg.colorbar(hi,ax=ax)
        cb.set_label('Energy Deposited')

    _nicez(ax,params)
    ax.set_title('Total Energy Depostiion')
    ax.set_ylabel('altitude [km]')


def plotprodloss(prod,loss,params,st):
    """ plot production/loss vs. alttiude """

    fg = figure(figsize=(15,8))
    ax = fg.subplots(1,2,sharey=True)
    fg.suptitle(f' Volume Production/Loss Rates   {st}')

    for a,R,title in zip(ax,[prod,loss],('Production','Loss')):
        if R.ndim==2:
            a.set_title('Volume {title} Rates')
            try:
                hi=a.pcolormesh(R.values,
                                R.z_km.values,
                                masked_invalid(R.values),
                                norm=LogNorm()) #pcolormesh canNOT handle nan at all!
                cb=fg.colorbar(hi,ax=a)
                cb.set_label('[cm$^{-3}$ s$^{-1}$ eV$^{-1}$]',labelpad=0)
                a.set_xscale('log')
                a.set_xlabel('Beam Energy [eV]')
                ax[0].set_ylabel('altitude [km]')
                _nicez(a,params)
            except TypeError as e:
                logging.warning(f'prodloss plot error    {e}')
        elif R.ndim==3:
            a.set_title('Volume {title} Rates')
            for a,R in zip(ax,[prod,loss]):
                for r in R:
                    for s in r.T:
                        a.plot(s, s.z_km, label=s.reaction.item())

                a.set_xlabel('[cm$^{-3}$ s$^{-1}$ eV$^{-1}$]',labelpad=0)
                a.set_ylabel('altitude [km]')
                a.set_xscale('log')
                a.legend(loc='best')
                _nicez(a,params)

# %%
def writeplots(fg:figure, plotprefix:str, params:dict):
    """ Save Matplotlib plots to disk.

    inputs:
    ------

    fg: Matplotlib figure handle
    odir: directory to write image into e.g. for this particular simulation.
    plotprefix: stem of filename
    E0: beam energy (eV)
    method: format of image

    Some observations on image formats:

      * TIF was not faster and was 100 times the file size!
      * PGF is slow and big file,
      * RAW crashes
      * JPG no faster than PNG
    """
    if 'odir' not in params or not params['odir']:
        return

    odir = Path(params['odir']).expanduser()
    draw() #Must have this here or plot doesn't update in animation multiplot mode!

    cn = odir / (plotprefix + f'beam{params["E0"]:.0f}.{params["plotformat"]}')
    print('write',cn)
    fg.savefig(cn, bbox_inches='tight', dpi=dpi)  # this is slow and async



#!/usr/bin/env python
"""
Example of aurora using Stan Solomon's GLOW Auroral model
code wrapping in Python by Michael Hirsch
"""
from pathlib import Path
import logging
from datetime import datetime
import xarray
import numpy as np
from os import chdir
#
from sciencedates import datetime2yd, find_nearest
try:
    from gridaurora import readmonthlyApF107
    from gridaurora.zglow import glowalt
except ImportError:
    readmonthlyApF107 = glowalt = None
# Fortran GLOW
import glowfort
# path monkey patching
glowpath=Path(__file__).parents[1]
oldcwd = Path.cwd()

GRIDERR = 'gridaurora is required for this program.  pip install gridaurora.'


def runglowaurora(params:dict, z_km:np.ndarray=None) -> xarray.Dataset:
    """ Runs Fortran GLOW program and collects results in friendly arrays with metadata. """
    #%% (-2) check/process user inputs
    assert isinstance(params['flux'],(float,int,np.ndarray))
    assert isinstance(params['E0'],   (float,'float32',int))
    assert isinstance(params['t0'],   (datetime,str))
    assert isinstance(params['glat'], (float,int))
    assert isinstance(params['glon'], (float,int))
#%% (-1) if no manual f10.7 and ap, autoload by date
    if not 'f107a' in params or params['f107a'] is None:
        if readmonthlyApF107 is None:
            raise ImportError(GRIDERR)
        f107Ap = readmonthlyApF107(params['t0'])
        params['f107a'] = params['f107p'] = f107Ap.loc['f107s'].item()
        params['f107']  = f107Ap.loc['f107o'].item()
        params['Ap']    = (f107Ap.loc['Apo'].item(),)*7

    chdir(glowpath) #FIXME: hack for path issue
#%% flux grid / date
    eflux = np.atleast_1d(params['flux'])

    yeardoy,utsec = datetime2yd(params['t0'])[:2]
#%% (0) define altitude grid [km]
    if z_km is None:
        if glowalt is not None:
            z_km = glowalt()
        else:
            raise ImportError(GRIDERR)
#%% (1) setup flux at top of ionosphere
    ener,dE = glowfort.egrid()

    if eflux.size == 1:
        logging.info('generating maxwellian input differential number flux spectrum')
        # maxwellian input PhiTop at top of ionosphere
        phitop = glowfort.maxt(eflux,params['E0'],ener, dE, itail=0, fmono=0, emono=0)
    elif eflux.size > 1: #eigenprofile generation, one non-zero bin at a time
        logging.info('running in eigenprofile mode')
        e0ind = find_nearest(ener,params['e0'])[0] # FIXME should we interpolate instead? Maybe not, as long as we're consistent ref. Semeter 2006
        phitop = np.zeros_like(ener)
        phitop[e0ind] = ener[e0ind] #value in glow grid closest to zett grid
    else:
        return ValueError('I do not understand your electron flux input. Should be scalar or vector')

    phi = np.stack((ener,dE,phitop), 1)   # Nalt x 3
    assert phi.shape[1] == 3
#%% (2) msis,iri,glow model
    ion,ecalc,photI,ImpI,isr,prate,lrate,UV = glowfort.aurora(z_km,yeardoy,utsec,params['glat'],params['glon']%360,
                                                              params['f107a'],params['f107'],params['f107p'],params['Ap'],phi)

#%% (3) collect outputs
    zeta=glowfort.cglow.zeta.T #columns 11:20 are identically zero

    lamb=[3371., 4278., 5200., 5577., 6300., 7320., 10400., 3466., 7774., 8446., 3726.,
          1356., 1304., 1027., 989., 1900.]
    products=['nO+(2P)','nO+(2D)','nO+(4S)','nN+','nN2+','nO2+','nNO+','nO','nO2','nN2','nNO']

    sim = xarray.Dataset()

    sim['ver'] = xarray.DataArray(
                   np.concatenate((zeta[:,:11],UV.T), axis=1),
                   dims=['z_km','wavelength_nm'],
                    coords={'z_km':z_km, 'wavelength_nm':lamb})

    sim['photIon'] = xarray.DataArray(np.hstack((photI[:,None],ImpI[:,None],ecalc[:,None],ion)),
                               dims=['z_km','type'],
                               coords={'z_km':z_km,
                                'type':['photoIoniz','eImpactIoniz','ne']+products})

    sim['isr'] = xarray.DataArray(isr,
                          dims=['z_km','param'],
                          coords={'z_km':z_km,'param':['ne','Te','Ti']})

    sim['phitop'] = xarray.DataArray(phi[:,2],
                           dims=['eV'],
                           coords={'eV':phi[:,0]})

    sim.attrs['sza'] = np.degrees(glowfort.cglow.sza)

    sim['tez'] = xarray.DataArray(glowfort.cglow.tez,
                           dims=['z_km'], coords={'z_km':z_km})


#%% production and loss rates
    prate = prate.T; lrate=lrate.T #fortran to C ordering 2x170x20, only first 12 columns are used

    #column labels by inspection of fortran/gchem.f staring after "DO 150 I=1,JMAX" (thanks Stan!)
    sim['prates'] = xarray.DataArray(prate[1,:,:12], #columns 12:20 are identically zero
                      dims=['z_km','reaction'],
                      coords={'z_km':z_km,
                              'reaction':['O+(2P)','O+(2D)','O+(4S)','N+','N2+','O2+','NO+',
                                 'N2(A)','N(2P)','N(2D)','O(1S)','O(1D)']}
                    )

    sim['lrates'] = xarray.DataArray(lrate[1,:,:12], #columns 12:20 are identically zero
                      dims=['z_km','reaction'],
                      coords={'z_km':z_km,
                        'reaction':['O+(2P)','O+(2D)','O+(4S)','N+','N2+','O2+','NO+',
                                 'N2(A)','N(2P)','N(2D)','O(1S)','O(1D)']}
                    )

    sim['sion'] = xarray.DataArray(glowfort.cglow.sion,
                         dims=['gas','z_km'],
                         coords={'gas':['O','O2','N2'],
                                 'z_km':z_km})
# %% cleanup
    chdir(oldcwd)

    return sim


def rundayglow(t0,glat,glon,f107a,f107,f107p,ap, conj=True):
    '''
    Run GLOW for no auroral input, to simulate Dayglow.

    conj = whether to account for photoelectrons from conjugate hemisphere

    After running, extra variables can be found in glowfort.cglow. Unfortunately
    you have to dig into the Fortran code to see what they mean.
    '''

#%% (-2) check/process user inputs
    assert isinstance(t0,   (datetime,str))
    assert isinstance(glat, (float,int))
    assert isinstance(glon, (float,int))

    chdir(glowpath) #FIXME: hack for path issue

#%% flux grid / date

    yd,utsec = datetime2yd(t0)[:2]
#%% (0) define altitude grid [km]
    #z = glowalt()
    z = np.concatenate((range(30,110,1),np.logspace(np.log10(110),np.log10(1200),90)))

#%% (1) setup external flux at top of ionosphere. Set it to zero. Photoelectron flux from
#       conjugate hemisphere will be calculated internally, if conj=True.
    ener,dE = glowfort.egrid()
    phitop = np.zeros_like(ener)
    phi = np.hstack((ener[:,None],dE[:,None],phitop[:,None]))

#%% (2) msis,iri,glow model
    iconj = int(conj) # convert boolean to int for passing to Fortran.
    ion,ecalc,photI,ImpI,isr,UV = glowfort.dayglow(z,yd,utsec,glat,glon%360,
                                             f107a,f107,f107p,ap,phi,iconj)

#%% handle the outputs including common blocks
    zeta=glowfort.cglow.zeta.T #columns 11:20 are identically zero

    lamb=[3371., 4278., 5200., 5577., 6300., 7320., 10400., 3466., 7774., 8446., 3726.,
          1356., 1304., 1027., 989., 1900.]
    products=['nO+(2P)','nO+(2D)','nO+(4S)','nN+','nN2+','nO2+','nNO+','nO','nO2','nN2','nNO']

    sim = xarray.Dataset()

    sim['ver'] = xarray.DataArray(np.concatenate((zeta[:,:11],UV.T), axis=1),
                   dims=['z_km','wavelength_nm'],
                    coords={'z_km':z,
                            'wavelength_nm':lamb})

    sim['photIon'] = xarray.DataArray(dims=['z_km','type'],
                        coords={'z_km':z,
                                'type':['photoIoniz','eImpactIoniz','ne']+products},
                   data=np.hstack((photI[:,None],ImpI[:,None],ecalc[:,None],ion)))

    sim['isr'] = xarray.DataArray(dims=['z_km','param'],
                        coords={'z_km':z,'param':['ne','Te','Ti']},
                         data=isr)

    sim['phitop'] = xarray.DataArray(dims=['eV'],
                       coords={'eV':phi[:,0]},
                       data=phi[:,2])

    sim['zceta'] = xarray.DataArray(dims=['z_km','wavelength_nm','type'],
                      coords={'z_km':z, 'wavelength_nm':lamb[:11]},
                    data=glowfort.cglow.zceta.T[:,:11,:])  #Nalt x Nwavelengths  xNproductionEmissions


    sim.attrs['sza'] = np.degrees(glowfort.cglow.sza)

    sim['tez'] = xarray.DataArray(dims=['z_km'],
                    coords={'z_km':z},
                       data=glowfort.cglow.tez)

    sim['sion'] = xarray.DataArray(dims=['gas','z_km'],
                    coords={'gas':['O','O2','N2'],
                             'z_km':z},data=glowfort.cglow.sion)


    chdir(oldcwd)

    return sim


def verprodloss(params:dict):
    """ for a single time, computes VER, production, and loss vs. unit input flux
    inputs:
    -------
    t: a single datetime() when the eigenprofiles should be computed (solar zenith angle computed in Fortran code)
    glatlon: geographic coordinates of site (magnetic coordinates computed in Fortran code)
    flux: a vector of scaled differential number flux for each energy bin, scaled to "unit" energy flux for your eigenprofile input
    EK: a vector of energies [eV] to compute unit responses

    """
    simparams = ('prates','lrates','ver','tez')
    if isinstance(params['t0'],(list,tuple)):
        params['t0'] = params['t0'][0]
        print('using time',params['t0'])

    assert isinstance(params['t0'], datetime)

    EK = np.atleast_1d(params['EK'])

    sim = xarray.Dataset(coords={'EK':EK})

    for e in EK:
        print(f'{params["t0"]} E0: {e:.0f}')
        params['E0'] = e

        s = runglowaurora(params)

        if len(sim.variables)==1:
            for p in simparams:
                sim[p] = s[p].expand_dims('eV')
            sim.attrs['sza'] = s.sza
        else:
            for p in simparams:
                sim[p].loc[e,...] = s[p].expand_dims('eV')

        #plotaurora(sim)

    return sim


def ekpcolor(eigen):
    """ Loads electron energy bins from .csv file.
    This would be the gridding that a Transcar simulation used, so that Glow and Transcar
    are run on the same energy bins.
    """

    if isinstance(eigen, xarray.DataArray):
        e0 = eigen.loc[:,'low'].values
        eEnd = eigen.loc[:,'high'][-1]
        diffnumflux = eigen.loc[:,'flux'].values
    elif isinstance(eigen, (str,Path)):
        eigen = Path(eigen).expanduser()
        if eigen.suffix == '.csv':
            e0 =   np.loadtxt(eigen, usecols=[0], delimiter=',')
            eEnd = np.loadtxt(eigen, usecols=[1], delimiter=',')[-1]
            diffnumflux = None
        elif eigen.suffix == '.h5':
            import pandas
            bins = pandas.read_hdf(eigen)
            e0 = bins['low'].values
            eEnd = bins['high'].iloc[-1]
            diffnumflux = bins['flux'].values
        else:
            raise ValueError(f'I do not understand what file you want me to read {eigen}')
    else:
        raise ValueError(f'unknown data type {type(eigen)}')

    return np.append(e0,eEnd),e0,diffnumflux


def makeeigen(params:dict):
    """ Make eigenprofiles on an energy grid. Like Dahlgren 2012, Hirsch 2015, etc. """

    #for t in T:
    sim = verprodloss(params)

#TODO time stack

    return sim



#!/usr/bin/env python
"""
Trivial example of aurora using Stan Solomon's GLOW Auroral model
code wrapping in Python by Michael Hirsch
"""
from matplotlib.pyplot import figure
from pandas import DataFrame
from numpy import hstack,arange,append,array,rollaxis
from os import chdir
try:
    import seaborn
except ImportError:
    pass
#
from sciencedates import datetime2yd
from pyiri90.runiri90 import runiri
from msise00.runmsis import rungtd1d
import glowaurora
from glowaurora import glowfort
#
glowpath=glowaurora.__path__[0]

def runglowaurora(eflux,e0,dt,glat,glon,f107a,f107,f107p,ap,mass):
    chdir(glowpath)
    yd,utsec = datetime2yd(dt)[:2]

    z = arange(80,110+1,1)
    z = append(z,array([111.5,113.,114.5,116.,118.,120.,122.,124.,126., 128.,130.,132.,134.,136.,138.,140.,142.,144.,146., 148.,150.,153.,156.,159.,162.,165.,168.,172.,176., 180.,185.,190.,195.,200.,205.,211.,217.,223.,230.,237.,244.,252.,260.,268.,276.,284.,292.,300.,309., 318.,327.,336.,345.,355.,365.,375.,385.,395.,406., 417.,428.,440.,453.,467.,482.,498.,515.,533.,551., 570.,590.,610.,630.,650.,670.,690.,710.,730.,750., 770.,790.,810.,830.,850.,870.,890.,910.,930.,950.]))

    glowfort.cglow.zz = z*1e5
    glowfort.cglow.znd[:]=0.

# Set other parameters and switches:
    glowfort.cglow.jlocal = 0
    glowfort.cglow.kchem = 4
    glowfort.cglow.iscale = 1
    glowfort.cglow.xuvfac = 3.
    glowfort.cglow.hlybr = 0.
    glowfort.cglow.fexvir = 0.
    glowfort.cglow.hlya = 0.
    glowfort.cglow.heiew = 0.
#%% (1) setup flux at top of ionosphere
    ener,dE = glowfort.egrid()

    phitop = glowfort.maxt(eflux,e0,ener, dE, itail=0, fmono=0, emono=0)

    phi = hstack((ener[:,None],dE[:,None],phitop[:,None]))

    glowfort.cglow.zz = z*1e5
    glowfort.cglow.znd[:]=0.
#%% (2) call MSIS
    tselecopts = (1,)*25
    dens,temp=rungtd1d(dt,z,glat,glon,f107a,f107,ap,mass,tselecopts)
    glowfort.cglow.zo = dens['O']
    glowfort.cglow.zn2 = dens['N2']
    glowfort.cglow.zo2 = dens['O2']
    glowfort.cglow.zrho = dens['Total']
    glowfort.cglow.zns  = dens['N']
    glowfort.cglow.ztn  = temp['heretemp']
#%% (3) call snoem
    """
    Call SNOEMINT to obtain NO profile from the Nitric Oxide Empirical
    Model (NOEM)
    """
    glowfort.cglow.zno = glowfort.snoemint(dt.strftime('%Y%j'),
                               glat,glon,f107,ap,z,temp['heretemp'])
#%% (4a) call iri-90
    outf,oarr = runiri(dt,z,glat,glon,f107,f107a,ap,mass=48)
    chdir(glowpath) #need this since iri90 changes path
#%% (4b) store iri90 in COMMON blocks, after unit conversion
    glowfort.cglow.ze = outf['ne']/1e6 # M-3 -> CM-3
    glowfort.cglow.ze[glowfort.cglow.ze<100.] = 100.

    glowfort.cglow.zti = outf['Ti']
    i = glowfort.cglow.zti<glowfort.cglow.ztn
    glowfort.cglow.zti[i] = glowfort.cglow.ztn[i]

    glowfort.cglow.zte = outf['Te']
    i = glowfort.cglow.zte<glowfort.cglow.ztn
    glowfort.cglow.zte[i] = glowfort.cglow.ztn[i]

    glowfort.cglow.zxden[2,:] = outf['nO+']/1e6
    glowfort.cglow.zxden[5,:] = outf['nO2+']/1e6
    glowfort.cglow.zxden[6,:] = outf['nNO+']/1e6
#%% glow model

    ion,ecalc,photI,ImpI,isr = glowfort.aurora(z,yd,utsec,glat,glon%360,
                                             f107a,f107,phi)
#%% handle the outputs including common blocks
    zeta=glowfort.cglow.zeta.T #columns 11:20 are identically zero

    ver = DataFrame(index=z,
                    data=zeta[:,:11],
                    columns=[3371, 4278, 5200, 5577, 6300,7320,10400,3466,
                             7774, 8446,3726])
    photIon = DataFrame(index=z,
                   data=hstack((photI[:,None],ImpI[:,None],ecalc[:,None],ion)),
                    columns=['photoIoniz','eImpactIoniz','ne',
                    'nO+(2P)','nO+(2D)','nO+(4S)','nN+','nN2+','nO2+','nNO+',
                    'nO','nO2','nN2','nNO'])

    isrparam = DataFrame(index=z,
                         data=isr,
                         columns=['ne','Te','Ti'])

    phitop = DataFrame(index=phi[:,0], #eV
                       data=phi[:,2],  #diffnumflux
                       columns=['diffnumflux'])
    zceta = glowfort.cglow.zceta.T

    return ver,photIon,isrparam,phitop,zceta
#%% plot
def plotaurora(phitop,ver,zceta,photIon,isr,dtime,glat,glon):
#%% incident flux at top of ionosphere
    ax = figure().gca()
    ax.plot(phitop.index,phitop['diffnumflux'])
    ax.set_title('Incident Flux',fontsize='x-large')
    ax.set_xlabel('Beam Energy [eV]',fontsize='large')
    ax.set_ylabel('Flux',fontsize='large')
    ax.set_xscale('log')
    ax.set_yscale('log')
    ax.set_ylim(bottom=1e-4)
    ax.tick_params(axis='both',which='major',labelsize='medium')
#%% results of impacts
    fg = figure()
    axs = fg.subplots(1,4,sharey=True, figsize=(15,8))
    fg.suptitle('{} ({},{})'.format(dtime,glat,glon),fontsize='x-large')
    fg.tight_layout(pad=3.2, w_pad=0.3)

    ax = axs[0]
    ax.plot(ver.values,ver.index)
    ax.set_xlabel('VER',fontsize='large')
    ax.set_ylabel('altitude [km]',fontsize='large')
    ax.set_ylim(top=ver.index[-1],bottom=ver.index[0])
    ax.set_xscale('log')
    ax.set_xlim(left=1e-4)
    ax.legend(ver.columns)
    ax.set_title('Volume emission rate',fontsize='x-large')

    ax = axs[1]
    ax.plot(photIon[['photoIoniz','eImpactIoniz']],photIon.index)
    ax.set_xlabel('ionization',fontsize='large')
    ax.set_xscale('log')
    ax.set_xlim(left=1e-1)
    ax.legend(photIon.columns[:2])
    ax.set_title('Photo and e$^-$ impact ionization',fontsize='x-large')

    ax = axs[2]
    ax.semilogx(photIon[['ne','nO+(2P)','nO+(2D)','nO+(4S)','nN+','nN2+','nO2+','nNO+',
                    'nO','nO2','nN2','nNO']], photIon.index)
    ax.set_xlabel('Density',fontsize='large')
    ax.set_xscale('log')
    ax.set_xlim(left=1e-3)
    ax.legend(photIon.columns[2:])
    ax.set_title('Electron and Ion Densities',fontsize='x-large')

    ax = axs[3]
    ax.semilogx(isr[['Te','Ti']], isr.index)
    ax.set_xlabel('Temperature [K]',fontsize='large')
    ax.legend(isr.columns[1:])
    ax.set_title('Particle Temperature',fontsize='x-large')

    for a in axs:
        a.grid(True)
        a.tick_params(axis='both',which='major',labelsize='medium')

#%% total energy deposition vs. altitude
    fg = figure()
    axs = fg.subplots(1,2,sharey=True, figsize=(15,8))
    fg.suptitle('{} ({},{})'.format(dtime,glat,glon),fontsize='x-large')
    fg.tight_layout(pad=3.2, w_pad=0.3)

    ax = axs[0]
    tez = glowfort.cglow.tez
    ax.plot(tez,ver.index)
    ax.set_xscale('log')
    ax.set_xlim(left=1e-1)
    ax.set_ylim(top=ver.index[-1],bottom=ver.index[0])
    ax.set_xlabel('Energy Deposited',fontsize='large')
    ax.set_ylabel('Altitude [km]',fontsize='large')
    ax.set_title('Total Energy Depostiion',fontsize='x-large')
#%% e^- impact ionization rates from ETRANS
    ax = axs[1]
    sion = glowfort.cglow.sion
    sion = DataFrame(index=ver.index,data=sion.T,columns=['O','O2','N2'])
    ax.plot(sion,ver.index)
    ax.set_xscale('log')
    ax.set_xlim(left=1e-6)
    ax.set_xlabel('e$^-$ impact ioniz. rate',fontsize='large')
    ax.set_title('electron impact ioniz. rates',fontsize='x-large')
    #ax.legend(True)
#%% constituants of per-wavelength VER
#    zcsum = zceta.sum(axis=-1)

    ax = figure().gca()
    for zc in rollaxis(zceta,1):
        ax.plot(ver.index,zc)
    ax.set_xlabel('emission constituants',fontsize='large')
    #ax.legend(True)



#!/usr/bin/env python
install_requires = ['python-dateutil','numpy','xarray',
                   'sciencedates','gridaurora']
tests_require=['pytest','nose','coveralls']
# %%
from setuptools import find_packages
from numpy.distutils.core import setup,Extension
from glob import glob
from os.path import join

# f2py -m aurora -c egrid.f maxt.f glow.f vquart.f gchem.f ephoto.f solzen.f rcolum.f etrans.f exsect.f ssflux.f snoem.f snoemint.f geomag.f nrlmsise00.f qback.f fieldm.f aurora_sub.f

# FUTURE: iri90.f not included in f2py b/c we use the pyiri90 package
# FUTURE: nrlmsise00.f not included .... msise00 package
fortranfiles=['egrid.f','maxt.f','glow.f','rout.f',
              'vquart.f','gchem.f','ephoto.f','solzen.f','rcolum.f',
              'etrans.f','exsect.f','ssflux.f','snoem.f','snoemint.f',
              'geomag.f','qback.f','fieldm.f',
              'nrlmsise00.f','iri90.f',
              'aurora_sub.f','dayglow_sub.f']

root='fortran'

fortranpaths = [join(root,f) for f in fortranfiles]
fortdata = glob(join(root,'*.dat'))
iridata = glob(join('iri','*.asc')) #in pyiri90
#%% prelim
ext=[Extension(name='glowfort',
               sources=fortranpaths,
               f2py_options=['--quiet'],
               #extra_f77_compile_args=['-O0'],
               #['-finit-local-zero'] #not needed
)]
               #include_dirs=[root],
               #library_dirs=[root])]


# %%
setup(name='glowaurora',
      packages=find_packages(),
      version='1.2.1',
      author='Michael Hirsch, Ph.D.',
      description='Model of auroral and airglow emissions',
      long_description=open('README.rst').read(),
      url='https://github.com/scivision/glowaurora',
#      package_dir={'glowaurora': 'glowaurora'}, #not working
 #     package_data={'glowaurora': ['fortran/*.dat']}, #not working, use data_files
      include_package_data=True,
      ext_modules=ext,
      data_files=[('glowaurora',fortdata),
                  ('glowaurora/iri',iridata)
                  ], #must have data_files to copy *.dat to site-packages
      install_requires=install_requires,
      python_requires='>=3.6',
      tests_require=tests_require,
      extras_require={'plot':['matplotlib>=2.2','seaborn'],
                        'io':['h5py','astropy','pandas',],
                      'tests':tests_require},
        classifiers=[
      'Development Status :: 4 - Beta',
      'Environment :: Console',
      'Intended Audience :: Science/Research',
       'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
      'Operating System :: OS Independent',
      'Programming Language :: Python :: 3.6',
      'Topic :: Scientific/Engineering :: Atmospheric Science',
      ],
      )


